@@ -78,11 +78,13 @@ _delete_from_keyring (const McpAccountStorage *self,
       /* flag the whole account as purged */
       gchar *removed = g_strdup (account);
       g_hash_table_replace (amd->removed_accounts, removed, removed);
+      g_key_file_remove_group (amd->secrets, removed, NULL);
     }
   else
     {
       /* remember to forget this one param */
       g_key_file_set_value (amd->removed, account, key, "");
+      g_key_file_remove_key (amd->secrets, account, key, NULL);
     }
 }
 
@@ -187,7 +189,7 @@ _keyring_commit (const McpAccountStorage *self,
     {
       gsize j;
       gsize k;
-      GStrv keys = g_key_file_get_keys (amd->secrets, accts[i], &k, NULL);
+      GStrv keys = g_key_file_get_keys (amd->removed, accts[i], &k, NULL);
 
       if (keys == NULL)
         k = 0;
@@ -195,15 +197,21 @@ _keyring_commit (const McpAccountStorage *self,
       for (j = 0; j < k; j++)
         {
           KeyringSetData *ksd = g_slice_new0 (KeyringSetData);
+          const gchar *key = keys[j];
+
+          /* for compatibility with old gnome keyring code we must strip  *
+           * the param- prefix from the name before saving to the keyring */
+          if (g_str_has_prefix (key, "param-"))
+            key += strlen ("param-");
 
           ksd->account = g_strdup (accts[i]);
-          ksd->name = g_strdup (keys[j]);
+          ksd->name = g_strdup (key);
           ksd->set = FALSE;
 
           gnome_keyring_delete_password (&keyring_schema,
               _keyring_set_cb, ksd, NULL,
               "account", accts[i],
-              "param", keys[j],
+              "param", key,
               NULL);
         }
 
@@ -411,6 +419,10 @@ _set (const McpAccountStorage *self,
   /* if we have a keyring, secrets are segregated */
   secret = mcp_account_manager_parameter_is_secret (am, account, key);
 
+  /* remove it from both sets, then re-add it to the right one if non-null */
+  g_key_file_remove_key (amd->secrets, account, key, NULL);
+  g_key_file_remove_key (amd->keyfile, account, key, NULL);
+
   if (val != NULL)
     {
       if (secret)
@@ -418,13 +430,6 @@ _set (const McpAccountStorage *self,
       else
         g_key_file_set_value (amd->keyfile, account, key, val);
     }
-  else
-    {
-      if (secret)
-        g_key_file_remove_key (amd->secrets, account, key, NULL);
-      else
-        g_key_file_remove_key (amd->keyfile, account, key, NULL);
-    }
 
   /* if we removed the account before, it now exists again, so... */
   g_hash_table_remove (amd->removed_accounts, account);
@@ -561,8 +566,10 @@ _delete (const McpAccountStorage *self,
           g_key_file_remove_group (amd->keyfile, account, NULL);
           _delete_from_keyring (self, am, account, NULL);
         }
-      else if (mcp_account_manager_parameter_is_secret (am, account, key))
+      else
         {
+          /* always delete from keyring, even if we didn't previously
+           * think it was secret - we might have been wrong */
           _delete_from_keyring (self, am, account, key);
         }
 
