@@ -38,7 +38,6 @@
 #include <dlfcn.h>
 #include <glib.h>
 #include <glib/gprintf.h>
-#include <glib/gi18n.h>
 
 #include <dbus/dbus-glib-lowlevel.h>
 
@@ -849,7 +848,7 @@ mcd_dispatcher_client_registry_ready_cb (McdClientRegistry *clients,
     }
 
     g_ptr_array_foreach (vas, (GFunc) g_value_array_free, NULL);
-    g_ptr_array_free (vas, TRUE);
+    g_ptr_array_unref (vas);
 }
 
 static void
@@ -903,7 +902,7 @@ _mcd_dispatcher_dispose (GObject * object)
         tp_clear_object (&priv->clients);
     }
 
-    tp_clear_pointer (&priv->connections, g_hash_table_destroy);
+    tp_clear_pointer (&priv->connections, g_hash_table_unref);
     tp_clear_object (&priv->master);
     tp_clear_object (&priv->dbus_daemon);
 
@@ -940,7 +939,7 @@ mcd_dispatcher_update_client_caps (McdDispatcher *self,
     }
 
     g_ptr_array_foreach (vas, (GFunc) g_value_array_free, NULL);
-    g_ptr_array_free (vas, TRUE);
+    g_ptr_array_unref (vas);
 }
 
 static void
@@ -2126,7 +2125,7 @@ _mcd_dispatcher_add_connection (McdDispatcher *self,
         _mcd_connection_start_dispatching (connection, vas);
 
         g_ptr_array_foreach (vas, (GFunc) g_value_array_free, NULL);
-        g_ptr_array_free (vas, TRUE);
+        g_ptr_array_unref (vas);
     }
     /* else _mcd_connection_start_dispatching will be called when we're ready
      * for it */
@@ -2585,6 +2584,13 @@ delegate_channels_cb (TpClient *client,
     DelegateChannelsCtx *ctx = to_delegate->ctx;
     McdClientProxy *clt_proxy = MCD_CLIENT_PROXY (client);
 
+    /* If the delegation succeeded, the channel has a new handler. If
+     * the delegation failed, the channel still has the old
+     * handler. Either way, the channel still has a handler, so it has
+     * been successfully dispatched (from 'handler invoked'). */
+    _mcd_channel_set_status (to_delegate->channel,
+        MCD_CHANNEL_STATUS_DISPATCHED);
+
     if (error != NULL)
       {
         DEBUG ("Handler refused delegated channels");
@@ -2616,6 +2622,9 @@ try_delegating (ChannelToDelegate *to_delegate)
     McdClientProxy *client;
     GList *channels = NULL;
 
+    DEBUG ("%s",
+        mcd_channel_get_object_path (to_delegate->channel));
+
     if (g_queue_get_length (to_delegate->handlers) == 0)
       {
         GValueArray *v;
@@ -2642,13 +2651,17 @@ try_delegating (ChannelToDelegate *to_delegate)
             g_strdup (mcd_channel_get_object_path (to_delegate->channel)),
             v);
 
+        DEBUG ("...but failed to delegate it: %s",
+            mcd_channel_get_object_path (to_delegate->channel),
+            to_delegate->error->message);
+
         delegation_done (to_delegate);
         return;
       }
 
     client = g_queue_pop_head (to_delegate->handlers);
 
-    DEBUG ("Try delegating channels to %s", _mcd_client_proxy_get_unique_name (
+    DEBUG ("...trying client %s", _mcd_client_proxy_get_unique_name (
         client));
 
     channels = g_list_prepend (channels, to_delegate->channel);
@@ -2722,6 +2735,8 @@ dispatcher_delegate_channels (
     guint i;
     GList *l;
 
+    DEBUG ("called");
+
     if (!check_preferred_handler (preferred_handler, &error))
         goto error;
 
@@ -2813,10 +2828,17 @@ error:
 static void
 present_handle_channels_cb (TpClient *client,
     const GError *error,
-    gpointer user_data G_GNUC_UNUSED,
+    gpointer user_data,
     GObject *weak_object)
 {
     DBusGMethodInvocation *context = user_data;
+    McdChannel *mcd_channel = MCD_CHANNEL (weak_object);
+
+    /* Whether presenting the channel succeeded or failed, the
+     * channel's handler hasn't been altered, so it must be set back
+     * to the dispatched state (from 'handler invoked'). */
+    _mcd_channel_set_status (mcd_channel,
+        MCD_CHANNEL_STATUS_DISPATCHED);
 
     if (error != NULL)
       {
@@ -2885,7 +2907,7 @@ dispatcher_present_channel (
 
     _mcd_client_proxy_handle_channels (client, -1, channels,
         user_action_time, NULL, present_handle_channels_cb,
-        context, NULL, NULL);
+        context, NULL, G_OBJECT (mcd_channel));
 
     g_list_free (channels);
     return;
