@@ -53,13 +53,10 @@
 #include "mcd-signals-marshal.h"
 #include "mcd-dispatcher.h"
 #include "mcd-dispatcher-context.h"
-#include "mcd-account-compat.h"
 #include "mcd-connection.h"
 #include "mcd-misc.h"
 #include "mcd-service.h"
 
-#include <libmcclient/mc-errors.h>
-
 /* DBus service specifics */
 #define MISSION_CONTROL_DBUS_SERVICE "org.freedesktop.Telepathy.MissionControl5"
 
@@ -196,3 +193,9 @@ mcd_service_run (McdService * self)
     g_main_loop_run (self->main_loop);
 }
 
+void
+mcd_service_stop (McdService * self)
+{
+    if (self->main_loop != NULL)
+        g_main_loop_quit (self->main_loop);
+}
